/**
 * Copyright (c) 2010 Perforce Software. All rights reserved.
 */
package com.perforce.maven.scm.provider.p4.command.remove;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.maven.scm.ScmException;
import org.apache.maven.scm.ScmFileSet;
import org.apache.maven.scm.ScmResult;
import org.apache.maven.scm.command.add.AddScmResult;
import org.apache.maven.scm.command.remove.AbstractRemoveCommand;
import org.apache.maven.scm.provider.ScmProviderRepository;

import com.perforce.maven.scm.provider.p4.command.P4ClientExecutor;
import com.perforce.maven.scm.provider.p4.command.P4Command;
import com.perforce.maven.scm.provider.p4.command.P4Executor;
import com.perforce.maven.scm.provider.p4.repository.P4ScmProviderRepository;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.option.client.DeleteFilesOptions;

/**
 * The class uses the Perforce delete command to open existing files for
 * deletion from the depot.
 */
public class P4RemoveCommand extends AbstractRemoveCommand implements P4Command {

    /** The Constant command. */
    public static final String command = "Perforce delete command";

    /**
     * @see org.apache.maven.scm.command.remove.AbstractRemoveCommand#
     *      executeRemoveCommand(org.apache.maven.scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet, java.lang.String)
     */
    protected ScmResult executeRemoveCommand(ScmProviderRepository repo,
            ScmFileSet files, String message) throws ScmException {
        P4RemoveResult p4Result = new P4RemoveResult();
        P4ScmProviderRepository p4Repo = (P4ScmProviderRepository) repo;
        P4ClientExecutor executor = new P4ClientExecutor(p4Repo, files,
                getLogger());
        List<IFileSpec> fileSpecs = null;
        List<String> filePaths = new ArrayList<String>();
        List<File> fileList = files.getFileList();
        if (fileList != null) {
            for (File f : fileList) {
                if (!f.getName().contentEquals(".")) {
                    filePaths.add(P4Executor.encodeWildcards(f
                            .getAbsolutePath()));
                }
            }
        }
        fileSpecs = FileSpecBuilder.makeFileSpecList(filePaths
                .toArray(new String[filePaths.size()]));
        try {
            DeleteFilesOptions deleteFilesOptions = new DeleteFilesOptions();
            List<IFileSpec> retFileSpecs = executor.getClient().deleteFiles(
                    fileSpecs, deleteFilesOptions);

            p4Result.process(retFileSpecs);
            p4Result.processRemovals(retFileSpecs);

        } catch (P4JavaException e) {
            if (getLogger().isErrorEnabled()) {
                getLogger().error(e);
            }
            throw new ScmException(e.getLocalizedMessage(), e);
        }
        List<String> removals = p4Result.getRemovals();
        return new AddScmResult(command, removals);
    }
}
